@@ -7,9 +7,10 @@ import logging
 import os
 import re
 import shlex
+import shutil
 import uuid
 
-from distutils.spawn import find_executable
+from math import floor
 from queue import Queue
 from typing import Any
 from typing import Callable
@@ -31,11 +32,13 @@ from ..ui_framework import CursesLinePart
 from ..ui_framework import CursesLines
 from ..ui_framework import Interaction
 from ..ui_framework import dict_to_form
-from ..ui_framework.form_utils import form_to_dict
+from ..ui_framework import form_to_dict
+from ..ui_framework import warning_notification
 
 
 from ..utils import abs_user_path
 from ..utils import human_time
+from ..utils import remove_ansi
 
 
 RESULT_TO_COLOR = [
@@ -51,7 +54,7 @@ get_color = lambda word: next(  # noqa: E731
 )
 
 
-def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> int:
+def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> Tuple[int, int]:
     # pylint: disable=too-many-branches
     """Find matching color for word
 
@@ -61,10 +64,11 @@ def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> int:
 
     colval = entry[colname]
     color = 0
+    decoration = 0
     if "__play_name" in entry:
         if not colval:
             color = 8
-        elif colname in ["__% completed", "__task_count", "__play_name"]:
+        elif colname in ["__task_count", "__play_name", "__progress"]:
             failures = entry["__failed"] + entry["__unreachable"]
             if failures:
                 color = 9
@@ -77,6 +81,9 @@ def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> int:
         else:
             color = get_color(colname[2:])
 
+        if colname == "__progress" and entry["__progress"].strip().lower() == "complete":
+            decoration = curses.A_BOLD
+
     elif "task" in entry:
         if entry["__result"].lower() == "__in_progress":
             color = get_color(entry["__result"])
@@ -90,7 +97,7 @@ def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> int:
         elif colname == "__duration":
             color = 12
 
-    return color
+    return color, decoration
 
 
 def content_heading(obj: Any, screen_w: int) -> Union[CursesLines, None]:
@@ -109,25 +116,13 @@ def content_heading(obj: Any, screen_w: int) -> Union[CursesLines, None]:
         detail = "PLAY [{play}:{tnum}] ".format(play=obj["play"], tnum=obj["__number"])
         stars = "*" * (screen_w - len(detail))
         heading.append(
-            tuple(
-                [
-                    CursesLinePart(
-                        column=0, string=detail + stars, color=curses.color_pair(0), decoration=0
-                    )
-                ]
-            )
+            tuple([CursesLinePart(column=0, string=detail + stars, color=0, decoration=0)])
         )
 
         detail = "TASK [{task}] ".format(task=obj["task"])
         stars = "*" * (screen_w - len(detail))
         heading.append(
-            tuple(
-                [
-                    CursesLinePart(
-                        column=0, string=detail + stars, color=curses.color_pair(0), decoration=0
-                    )
-                ]
-            )
+            tuple([CursesLinePart(column=0, string=detail + stars, color=0, decoration=0)])
         )
 
         if obj["__changed"] is True:
@@ -150,7 +145,7 @@ def content_heading(obj: Any, screen_w: int) -> Union[CursesLines, None]:
                     CursesLinePart(
                         column=0,
                         string=string,
-                        color=curses.color_pair(color),
+                        color=color,
                         decoration=curses.A_UNDERLINE,
                     )
                 ]
@@ -175,7 +170,7 @@ PLAY_COLUMNS = [
     "__ignored",
     "__in_progress",
     "__task_count",
-    "__% completed",
+    "__progress",
 ]
 
 TASK_LIST_COLUMNS = [
@@ -230,12 +225,30 @@ class Action(App):
         self._task_list_columns = task_list_columns
         self._content_key_filter = content_key_filter
 
-    def run_stdout(self) -> None:
+    @property
+    def mode(self):
+        """if mode == stdout and playbook artifact creation is enabled
+        run in interactive mode, but print stdout"""
+        if all(
+            (
+                self._args.mode == "stdout",
+                self._args.playbook_artifact_enable,
+                self._args.app != "replay",
+            )
+        ):
+            return "stdout_w_artifact"
+        return self._args.mode
+
+    def run_stdout(self) -> int:
         """Run in oldschool mode, just stdout
 
         :param args: The parsed args from the cli
         :type args: Namespace
         """
+        if self._args.app == "replay":
+            successful: bool = self._init_replay()
+            return 0 if successful else 1
+
         self._logger.debug("playbook requested in interactive mode")
         self._subaction_type = "playbook"
         self._logger = logging.getLogger(f"{__name__}_{self._subaction_type}")
@@ -247,10 +260,11 @@ class Action(App):
                     self.write_artifact()
                 self._logger.debug("runner finished")
                 break
+        return self.runner.ansible_runner_instance.rc
 
     def run(self, interaction: Interaction, app: AppPublic) -> Union[Interaction, None]:
         # pylint: disable=too-many-branches
-        """run :run or :load
+        """run :run or :replay
 
         :param interaction: The interaction from the user
         :type interaction: Interaction
@@ -267,12 +281,12 @@ class Action(App):
             self._logger = logging.getLogger(f"{__name__}_{str_uuid[-4:]}")
             self._name = f"run_{str_uuid[-4:]}"
             initialized = self._init_run()
-        elif interaction.action.match.groupdict().get("load"):
-            self._logger.debug("load requested in interactive mode")
-            self._subaction_type = "load"
-            self._name = "load"
+        elif interaction.action.match.groupdict().get("replay"):
+            self._logger.debug("replay requested in interactive mode")
+            self._subaction_type = "replay"
+            self._name = "replay"
             self._logger = logging.getLogger(f"{__name__}_{self._subaction_type}")
-            initialized = self._init_load()
+            initialized = self._init_replay()
 
         if not initialized:
             self._prepare_to_exit(interaction)
@@ -296,7 +310,7 @@ class Action(App):
                     break
 
             if self.steps.current.name == "quit":
-                if self._args.app == "load":
+                if self._args.app == "replay":
                     self._prepare_to_exit(interaction)
                     return self.steps.current
                 done = self._prepare_to_quit(self.steps.current)
@@ -350,26 +364,28 @@ class Action(App):
         self._logger.info("Run initialized and playbook started.")
         return True
 
-    def _init_load(self) -> bool:
-        """in the case of :load, load the artifact
+    def _init_replay(self) -> bool:
+        """in the case of :replay, replay the artifact
         check for a version, to be safe
         copy the calling app args as our our so the can be updated safely
         with a uuid attached to the name
         """
-        self._logger.debug("Starting load artifact request")
+        self._logger.debug("Starting replay artifact request with mode %s", self.mode)
 
-        self._update_args(
-            ["load"] + shlex.split(self._interaction.action.match.groupdict()["params_load"] or "")
-        )
+        if self.mode == "interactive":
+            self._update_args(
+                ["replay"]
+                + shlex.split(self._interaction.action.match.groupdict()["params_replay"] or "")
+            )
 
-        artifact_file = self._args.playbook_artifact_load
+        artifact_file = self._args.playbook_artifact_replay
 
-        if isinstance(self._args.playbook_artifact_load, str):
-            artifact_valid = os.path.exists(self._args.playbook_artifact_load)
+        if isinstance(self._args.playbook_artifact_replay, str):
+            artifact_valid = os.path.exists(self._args.playbook_artifact_replay)
         else:
             artifact_valid = False
 
-        if not artifact_valid:
+        if not artifact_valid and self.mode == "interactive":
             populated_form = self._prompt_for_artifact(artifact_file=artifact_file)
             if populated_form["cancelled"]:
                 return False
@@ -386,9 +402,17 @@ class Action(App):
         version = data.get("version", "")
         if version.startswith("1."):
             try:
-                self._plays.value = data["plays"]
-                self._interaction.ui.update_status(data["status"], data["status_color"])
-                self.stdout = data["stdout"]
+                stdout = data["stdout"]
+                if self.mode == "interactive":
+                    self._plays.value = data["plays"]
+                    self._interaction.ui.update_status(data["status"], data["status_color"])
+                    self.stdout = stdout
+                else:
+                    for line in data["stdout"]:
+                        if self._args.display_color is True:
+                            print(line)
+                        else:
+                            print(remove_ansi(line))
             except KeyError as exc:
                 self._logger.debug("missing keys from artifact file")
                 self._logger.debug("error was: %s", str(exc))
@@ -400,7 +424,7 @@ class Action(App):
             return False
 
         self._runner_finished = True
-        self._logger.debug("Completed load artifact request")
+        self._logger.debug("Completed replay artifact request with mode %s", self.mode)
         return True
 
     def _prompt_for_artifact(self, artifact_file: str) -> Dict[Any, Any]:
@@ -528,25 +552,54 @@ class Action(App):
     def _run_runner(self) -> None:
         """spin up runner"""
         executable_cmd: Optional[str]
+
+        if self.mode == "stdout_w_artifact":
+            mode = "interactive"
+        else:
+            mode = self.mode
+
+        if isinstance(self._args.set_environment_variable, dict):
+            set_envvars = {**self._args.set_environment_variable}
+        else:
+            set_envvars = {}
+
+        if self._args.display_color is False:
+            set_envvars["ANSIBLE_NOCOLOR"] = "1"
+
         kwargs = {
-            "cmdline": self._args.cmdline,
             "container_engine": self._args.container_engine,
+            "cwd": os.getcwd(),
             "execution_environment_image": self._args.execution_environment_image,
             "execution_environment": self._args.execution_environment,
             "inventory": self._args.inventory,
-            "navigator_mode": self._args.mode,
+            "navigator_mode": mode,
             "pass_environment_variable": self._args.pass_environment_variable,
-            "playbook": self._args.playbook,
-            "set_environment_variable": self._args.set_environment_variable,
+            "set_environment_variable": set_envvars,
         }
+
+        if isinstance(self._args.playbook, str):
+            kwargs.update({"playbook": self._args.playbook})
+
+        if isinstance(self._args.execution_environment_volume_mounts, list):
+            kwargs.update(
+                {"container_volume_mounts": self._args.execution_environment_volume_mounts}
+            )
+
         if self._args.execution_environment:
             executable_cmd = "ansible-playbook"
         else:
-            executable_cmd = find_executable("ansible-playbook")
+            executable_cmd = shutil.which("ansible-playbook")
             if not executable_cmd:
                 self._logger.error("'ansible-playbook' executable not found")
                 return
 
+        pass_through_arg = []
+        if self._args.help_playbook is True:
+            pass_through_arg.append("--help")
+        if isinstance(self._args.cmdline, list):
+            pass_through_arg.extend(self._args.cmdline)
+        kwargs.update({"cmdline": pass_through_arg})
+
         self.runner = CommandRunnerAsync(executable_cmd=executable_cmd, queue=self._queue, **kwargs)
         self.runner.run()
         self._runner_finished = False
@@ -565,6 +618,7 @@ class Action(App):
     def _handle_message(self, message: dict) -> None:
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-nested-blocks
+        # pylint: disable=too-many-statements
         """Handle a runner message
 
         :param message: The message from runner
@@ -578,10 +632,14 @@ class Action(App):
         else:
             if "stdout" in message and message["stdout"]:
                 self.stdout.extend(message["stdout"].splitlines())
+                if self.mode == "stdout_w_artifact":
+                    print(message["stdout"])
 
             if event in ["verbose", "error"]:
                 if "ERROR!" in message["stdout"]:
                     self._msg_from_plays = ("ERROR", 9)
+                    if self.mode == "interactive":
+                        self._notify_error(message["stdout"])
                 elif "WARNING" in message["stdout"]:
                     self._msg_from_plays = ("WARNINGS", 13)
 
@@ -643,12 +701,12 @@ class Action(App):
             self._plays.value[idx]["__task_count"] = task_count
             completed = task_count - self._plays.value[idx]["__in_progress"]
             if completed:
-                new = round((completed / task_count * 100))
+                new = floor((completed / task_count * 100))
                 current = self._plays.value[idx].get("__pcomplete", 0)
                 self._plays.value[idx]["__pcomplete"] = max(new, current)
-                self._plays.value[idx]["__% completed"] = str(max(new, current)) + "%"
+                self._plays.value[idx]["__progress"] = str(max(new, current)) + "%"
             else:
-                self._plays.value[idx]["__% completed"] = "0%"
+                self._plays.value[idx]["__progress"] = "0%"
 
     def _prepare_to_quit(self, interaction: Interaction) -> bool:
         """Looks like we're headed out of here
@@ -751,7 +809,11 @@ class Action(App):
         :param filename: The file to write to
         :type filename: str
         """
-        if filename or self._args.playbook_artifact_enable is True:
+        if (
+            filename
+            or self._args.playbook_artifact_enable is True
+            and self._args.help_playbook is not True
+        ):
             filename = filename or self._args.playbook_artifact_save_as
             filename = filename.format(
                 playbook_dir=os.path.dirname(self._args.playbook),
@@ -801,7 +863,18 @@ class Action(App):
                 self._logger.debug("Playbook rerun triggered")
             else:
                 self._logger.warning("Playbook rerun ignored, current playbook not complete")
-        elif self._subaction_type == "load":
+        elif self._subaction_type == "replay":
             self._logger.error("No rerun available when artifact is loaded")
         else:
             self._logger.error("sub-action type '%s' is invalid", self._subaction_type)
+
+    def _notify_error(self, message: str):
+        """show a blocking warning"""
+        warn_msg = ["Errors were encountered while running the playbook:"]
+        messages = remove_ansi(message).splitlines()
+        messages[-1] += "..."
+        warn_msg.extend(messages)
+        warn_msg += ["[HINT] To see the full error message try ':stdout'"]
+        warn_msg += ["[HINT] After it's fixed, try to ':rerun' the playbook"]
+        warning = warning_notification(warn_msg)
+        self._interaction.ui.show(warning)
