@@ -6,13 +6,15 @@ import curses
 import json
 import logging
 
-import os
 import re
 
+from collections.abc import Mapping
+
 from curses import ascii as curses_ascii
 
 from functools import lru_cache
-from math import ceil, floor
+from math import ceil
+from math import floor
 from typing import Any
 from typing import Callable
 from typing import Dict
@@ -24,8 +26,7 @@ from typing import Tuple
 from typing import Union
 
 from .colorize import Colorize
-from .colorize import rgb_to_ansi  # , hex_to_rgb_curses
-
+from .colorize import rgb_to_ansi
 
 from .curses_defs import CursesLine
 from .curses_defs import CursesLinePart
@@ -34,11 +35,16 @@ from .curses_defs import CursesLines
 from .curses_window import CursesWindow
 from .curses_window import Window
 
-from .form import Form
 from .field_text import FieldText
+from .form import Form
 from .form_handler_text import FormHandlerText
+from .form_utils import warning_notification
 from .menu_builder import MenuBuilder
 
+from .ui_config import UIConfig
+from ..utils import templar
+
+
 from ..yaml import human_dump
 
 STND_KEYS = {"^f/PgUp": "page up", "^b/PgDn": "page down", "\u2191\u2193": "scroll", "esc": "back"}
@@ -99,11 +105,11 @@ class UserInterface(CursesWindow):
     def __init__(
         self,
         screen_miny: int,
-        osc4: bool,
         kegexes: Callable[..., Any],
         refresh: int,
-        share_directory: str,
-        pbar_width: int = 11,
+        ui_config: UIConfig,
+        pbar_width: int = 8,
+        status_width=12,
     ) -> None:
         """init
 
@@ -113,12 +119,12 @@ class UserInterface(CursesWindow):
         :type pbar_width: int
         :param words_to_color: Words that get colored (regex, color_int)
         :type words_to_color: list
-        :param no_osc4: enable/disable osc4 terminal color change support
-        :type no_osc4: str (enabled/disabled)
         """
-        super().__init__()
-        self._color_menu_item: Callable[[int, str, Dict[str, Any]], int]
-        self._colorizer = Colorize(share_directory=share_directory)
+        super().__init__(ui_config=ui_config)
+        self._color_menu_item: Callable[[int, str, Dict[str, Any]], Tuple[int, int]]
+        self._colorizer = Colorize(
+            grammar_dir=self._ui_config.grammar_dir, theme_path=self._ui_config.theme_path
+        )
         self._content_heading: Callable[[Any, int], Union[CursesLines, None]]
         self._default_colors = None
         self._default_pairs = None
@@ -129,24 +135,20 @@ class UserInterface(CursesWindow):
         self._logger = logging.getLogger(__name__)
         self._menu_filter: Union[Pattern, None] = None
         self._menu_indicies: Tuple[int, ...] = tuple()
-        self._osc4 = osc4
 
         self._pbar_width = pbar_width
+        self._status_width = status_width
         self._prefix_color = 8
         self._refresh = [refresh]
         self._rgb_to_curses_color_idx: Dict[str, int] = {}
         self._screen_miny = screen_miny
         self._scroll = 0
-        self._theme_dir = os.path.join(share_directory, "themes")
         self._xform = self._default_obj_serialization
         self._status = ""
         self._status_color = 0
-
-        curses.curs_set(0)
-        self._set_colors()
         self._screen: Window = curses.initscr()
         self._screen.timeout(refresh)
-        self._one_line_input = FormHandlerText(screen=self._screen)
+        self._one_line_input = FormHandlerText(screen=self._screen, ui_config=self._ui_config)
 
     def clear(self) -> None:
         """clear the screen"""
@@ -264,7 +266,7 @@ class UserInterface(CursesWindow):
                 CursesLinePart(
                     column=col_starts[idx],
                     string=left,
-                    color=curses.color_pair(0),
+                    color=0,
                     decoration=curses.A_REVERSE,
                 )
             )
@@ -272,16 +274,23 @@ class UserInterface(CursesWindow):
                 CursesLinePart(
                     column=col_starts[idx] + len(left),
                     string=right,
-                    color=curses.color_pair(0),
+                    color=0,
                     decoration=0,
                 )
             )
         if self._status:
+            # place the status to the far right -1 for the scrollbar
+            # center place the uneven extra on the right, so flip it twice
+            status = self._status[0 : self._status_width - 1]  # max
+            status = status[::-1]  # reverse
+            status = status.center(self._status_width)  # pad
+            status = status[::-1]  # reverse
+            status = status.upper()  # upper
             footer.append(
                 CursesLinePart(
-                    column=self._screen_w - self._pbar_width,
-                    string=self._status[0 : self._pbar_width + 1].upper().center(self._pbar_width),
-                    color=curses.color_pair(self._status_color % self._number_colors),
+                    column=self._screen_w - self._status_width - 1,
+                    string=status,
+                    color=self._status_color,
                     decoration=curses.A_REVERSE,
                 )
             )
@@ -306,7 +315,7 @@ class UserInterface(CursesWindow):
         start_scroll_bar = body_start / menu_size * viewport_h
         stop_scroll_bar = body_stop / menu_size * viewport_h
         len_scroll_bar = ceil(stop_scroll_bar - start_scroll_bar)
-        color = curses.color_pair(self._prefix_color % self._number_colors)
+        color = self._prefix_color
         for idx in range(int(start_scroll_bar), int(start_scroll_bar + len_scroll_bar)):
             lineno = idx + len_heading
             line_part = CursesLinePart(
@@ -326,7 +335,7 @@ class UserInterface(CursesWindow):
         """
         self.disable_refresh()
         form_field = FieldText(name="one_line", prompt="")
-        clp = CursesLinePart(column=0, string=":", color=curses.color_pair(0), decoration=0)
+        clp = CursesLinePart(column=0, string=":", color=0, decoration=0)
         input_at = self._screen_h - 1  # screen y is zero based
         self._add_line(window=self._screen, lineno=input_at, line=tuple([clp]))
         self._screen.refresh()
@@ -453,19 +462,40 @@ class UserInterface(CursesWindow):
             if return_value is not None:
                 return return_value
 
-    def _action_match(self, entry: str) -> Union[Tuple[str, Action], Tuple[None, None]]:
-        """attempt to match the user input against the regexes
+    def _template_match_action(
+        self, entry: str, current: Any
+    ) -> Union[Tuple[str, Action], Tuple[None, None]]:
+        """attempt to template & match the user input against the regexes
         provided by each action
 
         :param entry: the user input
-        :type entry: str
+        :param current: the content on the screen
         :return: The name and matching action or not
         :rtype: str, Action or None, None
         """
+        if not entry.startswith("{{"):  # don't match pure template
+            if "{{" in entry and "}}" in entry:
+                if isinstance(current, Mapping):
+                    template_vars = current
+                    type_msgs = []
+                else:
+                    template_vars = {"this": current}
+                    type_msgs = ["Current content passed for templating is not a dictionary."]
+                    type_msgs.append("[HINT] Use 'this' to reference it (e.g. {{ this[0] }}")
+                errors, entry = templar(entry, template_vars)
+                if errors:
+                    msgs = ["Errors encountered while templating input"] + errors
+                    msgs.extend(type_msgs)
+                    self._show_form(warning_notification(msgs))
+                    return None, None
         for kegex in self._kegexes():
             match = kegex.kegex.match(entry)
             if match:
                 return kegex.name, Action(match=match, value=entry)
+
+        msgs = [f"Could not find a match for ':{entry}'"]
+        msgs.append("[HINT] Try ':help' for a list of available commands.")
+        self._show_form(warning_notification(msgs))
         return None, None
 
     def _serialize_color(self, obj: Any) -> CursesLines:
@@ -476,6 +506,7 @@ class UserInterface(CursesWindow):
         :return: The generated lines
         :rtype: CursesLines
         """
+
         if self.xform() == "source.ansi":
             return self._colorizer.render(doc=obj, scope=self.xform())
         if self.xform() == "source.yaml":
@@ -484,9 +515,13 @@ class UserInterface(CursesWindow):
             string = json.dumps(obj, indent=4, sort_keys=True)
         else:
             string = obj
-        colorized = self._colorizer.render(doc=string, scope=self.xform())
-        lines = self._color_lines_for_term(colorized)
-        return lines
+
+        scope = "no_color"
+        if self._ui_config.color:
+            scope = self.xform()
+
+        rendered = self._colorizer.render(doc=string, scope=scope)
+        return self._color_lines_for_term(rendered)
 
     def _color_lines_for_term(self, lines: List) -> CursesLines:
         """Give a list of dicts from tokenized lines
@@ -500,7 +535,7 @@ class UserInterface(CursesWindow):
         :return: the lines ready for curses
         :type: CursesLines
         """
-        if self._custom_colors_enabled:
+        if curses.COLORS > 16 and self._term_osc4_supprt:
             unique_colors = list(
                 set(chars["color"] for line in lines for chars in line if chars["color"])
             )
@@ -558,17 +593,17 @@ class UserInterface(CursesWindow):
         :rtype: CursesLinePart
         """
         if lp_dict["color"]:
-            if self._custom_colors_enabled:
+            if self._term_osc4_supprt and curses.COLORS > 16:
                 color = self._rgb_to_curses_color_idx[lp_dict["color"]]
             else:
                 red, green, blue = lp_dict["color"]
-                color = rgb_to_ansi(red, green, blue, self._number_colors)
+                color = rgb_to_ansi(red, green, blue, curses.COLORS)
         else:
             color = 0
         return CursesLinePart(
             column=lp_dict["column"],
             string=lp_dict["chars"],
-            color=curses.color_pair(color),
+            color=color,
             decoration=0,
         )
 
@@ -588,7 +623,7 @@ class UserInterface(CursesWindow):
         return heading, lines
 
     def _show_form(self, obj: Form) -> Form:
-        res = obj.present(screen=self._screen)
+        res = obj.present(screen=self._screen, ui_config=self._ui_config)
         return res
 
     def _show_obj_from_list(self, objs: List[Any], index: int, await_input: bool) -> Interaction:
@@ -621,7 +656,7 @@ class UserInterface(CursesWindow):
             first_line_idx = max(0, last_line_idx - (self._screen_h - 1 - heading_len - footer_len))
 
             if len(objs) > 1:
-                key_dict = {"+": "previous", "-": "next", "[0-9]": "goto"}
+                key_dict = {"-": "previous", "+": "next", "[0-9]": "goto"}
             else:
                 key_dict = {}
 
@@ -642,13 +677,15 @@ class UserInterface(CursesWindow):
             if entry == "KEY_RESIZE":
                 # only the heading knows about the screen_w and screen_h
                 heading = self._content_heading(objs[index], self._screen_w)
+                continue
 
             if entry == "_":
                 self._hide_keys = not self._hide_keys
                 heading, lines = self._filter_and_serialize(objs[index])
+                continue
 
             # get the less or more, wrap, incase we jumped out of the menu indices
-            elif entry == "-":
+            if entry == "-":
                 less = list(reversed([i for i in self._menu_indicies if i - index < 0]))
                 more = list(reversed([i for i in self._menu_indicies if i - index > 0]))
 
@@ -657,8 +694,9 @@ class UserInterface(CursesWindow):
                     index = ordered_indicies[0]
                     self.scroll(0)
                     entry = "KEY_F(5)"
+                continue
 
-            elif entry == "+":
+            if entry == "+":
                 more = [i for i in self._menu_indicies if i - index > 0]
                 less = [i for i in self._menu_indicies if i - index < 0]
 
@@ -667,24 +705,26 @@ class UserInterface(CursesWindow):
                     index = ordered_indicies[0]
                     self.scroll(0)
                     entry = "KEY_F(5)"
+                continue
 
-            elif entry.isnumeric():
+            if entry.isnumeric():
                 index = int(entry) % len(objs)
                 self.scroll(0)
                 entry = "KEY_F(5)"
+                continue
 
-            name, action = self._action_match(entry)
+            current = objs[index % len(objs)]
+
+            name, action = self._template_match_action(entry, current)
             if name and action:
                 if name == "refresh":
                     action = action._replace(value=index)
 
-                current = objs[index % len(objs)]
                 filtered = (
                     self._filter_content_keys(current)
                     if self._hide_keys and isinstance(current, dict)
                     else current
                 )
-
                 content = Content(showing=filtered)
                 return Interaction(name=name, action=action, content=content, ui=self._ui)
 
@@ -735,8 +775,9 @@ class UserInterface(CursesWindow):
         menu_builder = MenuBuilder(
             pbar_width=self._pbar_width,
             screen_w=self._screen_w,
-            number_colors=self._number_colors,
+            number_colors=curses.COLORS,
             color_menu_item=self._color_menu_item,
+            ui_config=self._ui_config,
         )
         menu_heading, menu_items = menu_builder.build(current, columns, indicies)
         return menu_heading, menu_items
@@ -789,7 +830,7 @@ class UserInterface(CursesWindow):
             if entry in ["KEY_RESIZE", "KEY_DOWN", "KEY_UP", "KEY_NPAGE", "KEY_PPAGE", "^F", "^B"]:
                 continue
 
-            name, action = self._action_match(entry)
+            name, action = self._template_match_action(entry, current)
             if name and action:
                 if name == "select":
                     if current:
@@ -808,7 +849,7 @@ class UserInterface(CursesWindow):
         columns: List = None,
         await_input: bool = True,
         filter_content_keys: Callable = lambda x: x,
-        color_menu_item: Callable = lambda *args, **kwargs: 0,
+        color_menu_item: Callable = lambda *args, **kwargs: (0, 0),
         content_heading: Callable = lambda *args, **kwargs: None,
     ) -> Union[Interaction, Form]:
         """Show something on the screen
